@@ -27,6 +27,7 @@ from typing import (
     Collection,
     Dict,
     List,
+    Literal,
     Optional,
     Tuple,
     Union,
@@ -35,7 +36,6 @@ from unittest.mock import Mock
 from urllib.parse import urlencode
 
 import pymacaroons
-from typing_extensions import Literal
 
 from twisted.test.proto_helpers import MemoryReactor
 from twisted.web.resource import Resource
